Package: RivRetrieve
Title: Retrieve Global River Gauge Data
Version: 0.2.0
Authors@R: 
    c(person("Ryan", "Riggs", ,  "ryanriggs7@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-6834-9469")),
      person("Simon", "Moulds", , "sim.moulds@gmail.com", role = c("aut"), comment = c(ORCID = "0000-0002-7297-482X")),
      person("Michel", "Wortmann", , "michel.wortmann@ouce.ox.ac.uk",role=c("aut"), comment = c(ORCID = "0000-0002-1879-7674")),
      person("Louise", "Slater", , "louise.slater@ouce.ox.ac.uk",role=c("aut"), comment = c(ORCID = "0000-0001-9416-488X")),
      person("George", "Allen", , "geoallen@vt.edu",role=c("aut"), comment = c(ORCID = "0000-0001-8301-5301"))
      )
Description: Provides access to global river gauge data from a variety of national-level river agencies. The package interfaces with the national-level agency websites to provide access to river gauge locations, river discharge, and river stage. Currently, the package is available for the following countries: Argentina, Australia, Brazil, Canada, Chile, France, Germany, Japan, South Africa, the United Kingdom, and the United States.
License: MIT + file LICENSE
URL: https://github.com/Ryan-Riggs/RivRetrieve
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1)
Imports: BBmisc, dataRetrieval, data.table, devtools, dplyr, httr,
        jsonlite, lubridate, readr, rlang, rlist, RSelenium, rvest,
        stringr, tibble, tidyhydat, tidyr
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-08-21 16:59:12 UTC; ryanr
Author: Ryan Riggs [aut, cre] (ORCID: <https://orcid.org/0000-0001-6834-9469>),
  Simon Moulds [aut] (ORCID: <https://orcid.org/0000-0002-7297-482X>),
  Michel Wortmann [aut] (ORCID: <https://orcid.org/0000-0002-1879-7674>),
  Louise Slater [aut] (ORCID: <https://orcid.org/0000-0001-9416-488X>),
  George Allen [aut] (ORCID: <https://orcid.org/0000-0001-8301-5301>)
Maintainer: Ryan Riggs <ryanriggs7@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-21 19:10:02 UTC
Built: R 4.5.2; ; 2026-08-22 00:18:49 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(original,rr_tbl)
S3method(plot,rr_tbl)
export(argentina)
export(australia)
export(brazil)
export(canada)
export(chile)
export(france)
export(germany)
export(ireland)
export(japan)
export(original)
export(quebec)
export(scotland)
export(southAfrica)
export(sweden)
export(uk)
export(usa)
import(RSelenium)
import(dataRetrieval)
import(devtools)
import(dplyr)
import(httr)
import(jsonlite)
import(lubridate)
import(rvest)
import(stringr)
import(tibble)
import(tidyhydat)
import(tidyr)
importFrom(BBmisc,is.error)
importFrom(data.table,fread)
importFrom(readr,locale)
importFrom(readr,read_delim)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlist,list.select)
importFrom(rlist,list.stack)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,time)
importFrom(utils,download.file)
importFrom(utils,read.csv)
importFrom(utils,unzip)

# RivRetrieve

[![metacran downloads](https://cranlogs.r-pkg.org/badges/grand-total/RivRetrieve)](https://cran.r-project.org/package=RivRetrieve) [![metacran downloads](https://cranlogs.r-pkg.org/badges/last-month/RivRetrieve)](https://cran.r-project.org/package=RivRetrieve) [![metacran downloads](https://cranlogs.r-pkg.org/badges/last-week/RivRetrieve)](https://cran.r-project.org/package=RivRetrieve)

An R package for locating and downloading global river gauge data for the following regions: Argentina, Australia, Brazil, Canada, Chile, France, Germany, Ireland, Japan, Quebec, Scotland, South Africa, Sweden, the United Kingdom, and the United States. Please see <https://doi.org/10.1088/1748-9326/acd407> for a detailed analysis on the gauge data.

[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.10125516.svg)](https://doi.org/10.5281/zenodo.10125516)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",warning=FALSE,message=FALSE,fig.width=7
)

## ----setup--------------------------------------------------------------------
library(RivRetrieve)
# Amazon River near Obidos, Brazil
## siteNumber='17050001'
## discharge=brazil(site=siteNumber,variable='discharge')
## stage=brazil(site=siteNumber,variable='stage')
## plot(discharge$Date,discharge$Q, type='l',xlab='',ylab='Discharge (cms)')

# Mississippi River near Baton Rouge, LA, USA
siteNumber="07374000"
discharge=usa(site=siteNumber,variable='discharge')
stage=usa(site=siteNumber,variable='stage')
plot(discharge$Date,discharge$Q, type='l',xlab='',ylab='Discharge (cms)')




## ----echo=TRUE----------------------------------------------------------------
library(knitr)
## kable(raw[1:5,],caption='Raw Brazilian gauge data')

raw=original(discharge)
kable(raw[1:5,],caption='Raw US gauge data')

## ----echo=TRUE----------------------------------------------------------------
## brazilSites=brazil(sites=TRUE)
## kable(brazilSites[1:10,],caption='Example Brazilian river gauge locations')

usaSites=usa(sites=TRUE)
kable(usaSites[1:10,],caption='Example US river gauge locations')

## -----------------------------------------------------------------------------
# Annual timeseries
## siteNumber='17050001'
## recent=brazil(site=siteNumber,variable='stage',start_date = start,end_date = end)
## plot(recent$Date,recent$H,type='l', xlab='',ylab='Stage (m)')

siteNumber="07374000"
start='2009-01-01'
end='2010-01-31'
recent=usa(site=siteNumber,variable='stage',start_date = start,end_date = end)
plot(recent$Date,recent$H,type='l', xlab='',ylab='Stage (m)')

#' @title france
#' @name france
#'
#' @description Retrieve French gauge data
#'
#' @param site French gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' df <- france('K027401001')
#' plot(df$Date, df$Q, type='l')
#' }
#' @export
france <- function(site,
                   variable = "discharge",
                   start_date = NULL,
                   end_date = NULL,
                   sites = FALSE,
                   ...) {

  if (sites) {
    return(french_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  original_data <- download_france_data(site, start_date, end_date)
  if(nrow(original_data)==0){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  data <- original_data %>%
    mutate(
      Date = as.Date(!!sym("date_obs_elab")),
      !!column_name := as.numeric(!!sym("resultat_obs_elab")) / 1000.
    ) %>%
    dplyr::select(all_of(c("Date", column_name))) %>%
    arrange(Date)
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

download_france_data <- function(site, start_date, end_date) {
  ## FIXME this will only download 20000 records at once, so we need to provide a method to split the download if necessary
  web <- paste0(
    "https://hubeau.eaufrance.fr/api/v2/hydrometrie/obs_elab?code_entite=",
    site,
    "&date_debut_obs_elab=", start_date,
    "&date_fin_obs_elab=", end_date,
    "&grandeur_hydro_elab=QmnJ",
    "&size=20000"
  )
  original_data <- fromJSON(web)$data
  original_data <- as_tibble(original_data)
  return(original_data)
}

context("Download functions")
library(testthat)
library(RivRetrieve)

test_that("Downloading site locations is working", {

  Australia_sites <- australia(sites = TRUE)
  expect_s3_class(Australia_sites,'data.frame')

  Brazil_sites <- brazil(sites = TRUE)
  expect_s3_class(Brazil_sites, "data.frame")

  Chile_sites <- chile(sites = TRUE)
  expect_s3_class(Chile_sites, "data.frame")

  France_sites <- france(sites = TRUE)
  expect_s3_class(France_sites, "data.frame")

  Japan_sites <- japan(sites = TRUE)
  expect_s3_class(Japan_sites, "data.frame")

  SouthAfrica_sites <- southAfrica(sites = TRUE)
  expect_s3_class(SouthAfrica_sites, "data.frame")

  UK_sites <- uk(sites = TRUE)
  expect_s3_class(UK_sites, "data.frame")

  USA_sites <- usa(sites = TRUE)
  expect_s3_class(USA_sites, "data.frame")

})

## TODO use httrtest, to avoid actually downloading data during tests

## test_that("Download stage functions are working", {

##   Australia_sites <- australia(sites = TRUE)
##   australia_df <- australia(Australia_sites$site[1], "stage")
##   expect_s3_class(australia_df, "data.frame")

##   Brazil_sites <- brazil(sites = TRUE)
##   brazil_df <- brazil(Brazil_sites$site[1], "stage")
##   expect_s3_class(brazil_df, "data.frame")

##   France_sites <- france(sites = TRUE)
##   france_df <- france('K027401001', "stage")
##   expect_s3_class(france_df, "data.frame")

##   SouthAfrica_sites <- southAfrica(sites = TRUE)
##   # southAfrica_df <- southAfrica("X3H023", "stage",start_date=as.Date('2005-01-01'),end_date=as.Date('2005-12-31'))
##   # expect_s3_class(southAfrica_df, "data.frame")

##   UK_sites <- uk(sites = TRUE)
##   uk_df <- uk(UK_sites$site[1], "stage",start_date=as.Date('2005-01-01'),end_date=as.Date('2005-12-31'))
##   expect_s3_class(uk_df, "data.frame")

##   USA_sites <- usa(sites = TRUE)
##   usa_df <- usa(USA_sites$site[10], "stage")
##   expect_s3_class(usa_df, "data.frame")
## })

## test_that("Download discharge functions are working", {

##   Australia_sites <- australia(sites = TRUE)
##   australia_df <- australia(Australia_sites$site[1], "discharge")
##   expect_s3_class(australia_df, "data.frame")

##   Brazil_sites <- brazil(sites = TRUE)
##   brazil_df <- brazil(Brazil_sites$site[1], "discharge")
##   expect_s3_class(brazil_df, "data.frame")

##   Chile_sites <- chile(sites = TRUE)
##   # chile_df <- chile(Chile_sites$site[1], "discharge")
##   # expect_s3_class(chile_df, "data.frame")

##   France_sites <- france(sites = TRUE)
##   france_df <- france('K027401001', "discharge")
##   expect_s3_class(france_df, "data.frame")

##   Japan_sites <- japan(sites = TRUE)
##   # japan_df <- japan(Japan_sites$site[1], "discharge",start_date=as.Date('2000-01-01'), end_date=as.Date('2001-12-31'))
##   # expect_s3_class(japan_df, "data.frame")

##   SouthAfrica_sites <- southAfrica(sites = TRUE)
##   # southAfrica_df <- southAfrica("X3H023", "discharge",start_date=as.Date('2005-01-01'),end_date=as.Date('2005-12-31'))
##   # expect_s3_class(southAfrica_df, "data.frame")

##   UK_sites <- uk(sites = TRUE)
##   uk_df <- uk(UK_sites$site[1], "discharge")
##   expect_s3_class(uk_df, "data.frame")

##   USA_sites <- usa(sites = TRUE)
##   usa_df <- usa(USA_sites$site[1], "discharge")
##   expect_s3_class(usa_df, "data.frame")
## })

## test_that("Download raw data are working", {

##   Australia_sites <- australia(sites = TRUE)
##   australia_df <- australia(Australia_sites$site[1], "discharge")
##   australia_raw=original(australia_df)
##   expect_s3_class(australia_raw, "data.frame")

##   Brazil_sites <- brazil(sites = TRUE)
##   brazil_df <- brazil(Brazil_sites$site[1], "discharge")
##   brazil_raw=original(brazil_df)
##   expect_s3_class(brazil_raw, "data.frame")

##   Chile_sites <- chile(sites = TRUE)
##   # chile_df <- chile(Chile_sites$site[1], "discharge")
##   # chile_raw=original(chile_df)
##   # expect_s3_class(chile_raw, "data.frame")

##   France_sites <- france(sites = TRUE)
##   france_df <- france('K027401001', "discharge")
##   france_raw=original(france_df)
##   expect_s3_class(france_raw, "data.frame")

##   Japan_sites <- japan(sites = TRUE)
##   # japan_df <- japan(Japan_sites$site[1], "discharge",start_date=as.Date('2000-01-01'), end_date=as.Date('2001-12-31'))
##   # japan_raw=original(japan_df)
##   # expect_s3_class(japan_raw, "data.frame")

##   SouthAfrica_sites <- southAfrica(sites = TRUE)
##   # southAfrica_df <- southAfrica("X3H023", "discharge",start_date=as.Date('2005-01-01'),end_date=as.Date('2005-12-31'))
##   # southAfrica_raw=original(southAfrica_df)
##   # expect_s3_class(southAfrica_raw, "data.frame")

##   UK_sites <- uk(sites = TRUE)
##   uk_df <- uk(UK_sites$site[1], "discharge")
##   uk_raw=original(uk_df)
##   expect_s3_class(uk_raw, "data.frame")

##   USA_sites <- usa(sites = TRUE)
##   usa_df <- usa(USA_sites$site[1], "discharge")
##   usa_raw=original(usa_df)
##   expect_s3_class(usa_raw, "data.frame")
## })

#' Get original data
#'
#' @param x Tibble.
#' @param ... Additional arguments. None implemented.
#'
#' @return list
#' @export
original <- function(x, ...) {
  UseMethod("original")
}

#' @export
original.rr_tbl <- function(x, ...) {
  return(attr(x, "original"))
}

#' Plot values
#'
#' @param x Tibble.
#' @param ... Additional arguments. None implemented.
#'
#' @return ggplot2
#' @export
plot.rr_tbl <- function(x, ...) {
  ## varname <- metadata(x)$VariableName
  ## units <- metadata(x)$UnitAbbreviation
  ## if (is.na(units) | nchar(units) == 0) {
  ##   units <- ""
  ## }
  ## y_label <- "TODO" #sprintf("%s [%s]", varname, units)
  ## title <- "TODO" #metadata(x)$citation
  ## p <- ggplot(data = x, aes_string(x = "Date", y = "Q")) +
  ##   geom_line() +
  ##   ylab(y_label) +
  ##   xlab("Time") +
  ##   ggtitle(title)
  ## p
  return(plot(x$Date, x$Q, ...))
}

#' @title chile
#' @name chile
#'
#' @description Retrieve Chilean gauge data
#'
#' @param site Chilean gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' df <- chile('01201005')
#' plot(df$Date, df$Q, type='l')
#' }
#' @export
chile <- function(site,
                  variable = "discharge",
                  start_date = NULL,
                  end_date = NULL,
                  sites = FALSE,
                  ...) {

  if (sites) {
    return(chile_sites)
  }
  if (variable == "stage") {
    stop("Stage data is not currently available for Chile")
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  original_data <- try(download_chile_data(site),silent=TRUE)
  if(is.error(original_data)==TRUE){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  data <- original_data %>%
    unite("Date", "agno", "mes", "dia", sep = "-") %>%
    mutate(
      Date = as.Date(!!sym("Date")),
      valor = as.numeric(!!sym("valor"))
    ) %>%
    rename(!!column_name := "valor") %>%
    arrange(.data$Date) %>%
    filter(.data$Date >= start_date & .data$Date <= end_date)
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

download_chile_data <- function(site) {
  original <- "https://explorador.cr2.cl/request.php?options={%22variable%22:{%22id%22:%22qflxDaily%22,%22var%22:%22caudal%22,%22intv%22:%22daily%22,%22season%22:%22year%22,%22stat%22:%22mean%22,%22minFrac%22:80},%22time%22:{%22start%22:-946771200,%22end%22:1631664000,%22months%22:%22A%C3%B1o%20completo%22},%22anomaly%22:{%22enabled%22:false,%22type%22:%22dif%22,%22rank%22:%22no%22,%22start_year%22:1980,%22end_year%22:2010,%22minFrac%22:70},%22map%22:{%22stat%22:%22mean%22,%22minFrac%22:10,%22borderColor%22:%227F7F7F%22,%22colorRamp%22:%22Jet%22,%22showNaN%22:false,%22limits%22:{%22range%22:[5,95],%22size%22:[4,12],%22type%22:%22prc%22}},%22series%22:{%22sites%22:[%22"
  ending <- "%22],%22start%22:null,%22end%22:null},%22export%22:{%22map%22:%22Shapefile%22,%22series%22:%22CSV%22,%22view%22:{%22frame%22:%22Vista%20Actual%22,%22map%22:%22roadmap%22,%22clat%22:-18.0036,%22clon%22:-69.6331,%22zoom%22:5,%22width%22:461,%22height%22:2207}},%22action%22:[%22export_series%22]}"
  ## Wait a short time before downloading, to prevent overloading the server
  Sys.sleep(.25)
  outpath <- tempfile()
  website <- paste0(original, site, ending)
  headers <- httr::add_headers(
    `User-Agent` = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36"
  )

  resp <- httr::RETRY("GET", website, headers, times = 3, quiet = TRUE)
  httr::stop_for_status(resp, task = "request export link from explorador.cr2.cl")
  file <- httr::content(resp, as = "text", encoding = "UTF-8")

  parsed <- tryCatch(jsonlite::fromJSON(file), error = function(e) NULL)
  if (is.null(parsed)) {
    stop("explorador.cr2.cl did not return valid JSON; response may have changed again.")
  }
  if (length(parsed$errors) > 0) {
    stop("explorador.cr2.cl reported an error: ", paste(parsed$errors, collapse = "; "))
  }
  rel_url <- parsed$export$series$url
  if (is.null(rel_url) || !nzchar(rel_url)) {
    stop("No export URL found in explorador.cr2.cl's response.")
  }
  page <- if (grepl("^https?://", rel_url)) {
    rel_url
  } else {
    paste0("https://explorador.cr2.cl/", sub("^/", "", rel_url))
  }

  dl_resp <- httr::RETRY("GET", page, headers, httr::write_disk(outpath, overwrite = TRUE),
                         times = 3, quiet = TRUE)
  httr::stop_for_status(dl_resp, task = "download Chile CSV export")

  original_data <- read_delim(outpath, show_col_types = FALSE)
  names(original_data) <- sub("\\s+", "", names(original_data))
  original_data <- as_tibble(original_data)
  return(original_data)
}

#' @title usa
#' @name usa
#'
#' @description Retrieve USA gauge data
#'
#' @param site USA gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' df <- usa("02471078", variable="discharge")
#' plot(df$Date, df$Q, type='l')
#' }
#' @export
usa <- function(site,
                variable = "stage",
                start_date = NULL,
                end_date = NULL,
                sites = FALSE,
                ...) {

  if (sites) {
    return(usa_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  if (variable == "stage") {
    param_code <- "00065"
    mult <- 0.3048 # ft -> m
  } else if (variable == "discharge") {
    param_code <- "00060"
    mult <- 0.02832 # ft3/s -> m3/s
  }
  original_data <- try(read_waterdata_daily(
    monitoring_location_id=paste0('USGS-',site),
    parameter_code=param_code,
    time=c(start_date,
    end_date),
    statistic_id = "00003",
    skipGeometry = TRUE
  ),silent=TRUE)
  if(is.error(original_data)==TRUE|length(original_data)==0|nrow(original_data)==0){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  original_data <- as_tibble(original_data)
  data <- original_data %>%
    dplyr::select(4, 5) %>%
    setNames(c("Date", "X")) %>%
    mutate(X = as.numeric(.data$X) * mult) %>%
    rename(!!column_name := "X")
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

#' @title southAfrica
#' @name southAfrica
#'
#' @description Retrieve South African gauge data
#'
#' @param site South African gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' site <- "X3H023"
#' start_date <- as.Date("2000-01-01")
#' end_date <- as.Date("2010-01-01")
#' x <- southAfrica(site, "stage", start_date, end_date)
#' }
#' @export
southAfrica <- function(site,
                        variable = "stage",
                        start_date = NULL,
                        end_date = NULL,
                        sites = FALSE,
                        ...) {

  if (sites) {
    return(southAfrican_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  original_data <- try(download_sa_data(
    site, variable, start_date, end_date, primary = FALSE
  ),silent=TRUE)
  if(is.error(original_data)==TRUE|length(original_data)==0){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  data <- original_data %>%
    mutate(DATE = as.Date(.data$DATE, format = "%Y%m%d"))
  if (variable == "stage") {
    data <- data %>%
      mutate(across(starts_with("COR.LEVEL"), as.numeric)) %>%
      rename(Date = "DATE") %>%
      group_by(.data$Date) %>%
      summarize(!!column_name := mean(.data$COR.LEVEL))
  } else {
    data <- data %>%
      rename(Date = "DATE", !!column_name := "COR.FLOW") %>%
      dplyr::select(all_of(c("Date", column_name)))
  }
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

construct_endpoint <- function(site, start_date, end_date) {
  chunk_start_date <- format(start_date, "%Y-%m-%d")
  chunk_end_date <- format(end_date, "%Y-%m-%d")
  endpoint <- paste0(
    "https://www.dws.gov.za/Hydrology/Verified/HyData.aspx?",
    "Station=", site, "100.00",
    "&DataType=Point",
    "&StartDT=", chunk_start_date,
    "&EndDT=", chunk_end_date,
    "&SiteType=RIV"
  )
  return(endpoint)
}

download_sa_data <- function(site,
                             variable,
                             start_date,
                             end_date,
                             primary) {
  ## Convert to date
  start_date=as.Date(start_date)
  end_date=as.Date(end_date)

  # ## divide timeseries into months, because we can only
  # ## scrape data one month at a time.
  # ts <- seq(start_date, end_date, by = "1 day")
  # years <- year(ts) %>%
  #   unique() %>%
  #   sort()
  # n_years <- length(years)
  # if (primary || (variable == "stage")) {
  #   data_type <- "Point"
  #   header <- c(
  #     "DATE", "TIME", "COR_LEVEL",
  #     "COR_LEVEL_QUAL", "COR_FLOW", "COR_FLOW_QUAL"
  #   )
  # } else {
  #   data_type <- "Daily"
  #   header <- c("DATE", "D_AVG_FR", "QUAL")
  # }
  # ## Number of data columns
  # n_cols <- length(header)
  # data_list <- list()
  endpoint = construct_endpoint(site,start_date,end_date)

  tmp = tempfile()
  download.file(endpoint,tmp)
  data=read_delim(tmp,skip=9,delim=' ')
  list_values=list()
  for(i in 2:nrow(data)){
    result = unlist(str_split(data$Level[i],' '))
    result = na.omit(ifelse(result=='',NA,result))
    list_values[[i]] = as.data.frame(matrix(result,ncol=4))
  }
  list_output=do.call('rbind',list_values)
  end_of_data = grep('http',list_output$V1)-1
  list_output = cbind(data[2:end_of_data,1:2],list_output[1:end_of_data,])
  headers=unlist(str_split(data[1,],' '))
  headers = na.omit(ifelse(headers==''|headers=='NA',NA,headers))
  colnames(list_output) = headers
  data=list_output
  colnames(data)[6] = 'QUA_2'
  # response <- GET(endpoint)
  #   data <- content(response) %>%
  #     html_element("body") %>%
  #     html_text("pre")
  # data <- str_split(data, '\n')
  # data <- unlist(data)
    ## Find out whether there is any data for
    ## the requested time period
    # header_row <- grep("^DATE", data)
    # if (length(header_row) == 0) {
    #   stop('This gauge does not have a record associated with it and/or the agency website is down.')
    # } else {
    #   header_row <- header_row[1]
    # }
    # data_rows <- grep("^[0-9]{8} ", data)
    # ## Convert to list
    # data <- as.list(data)
    # ## Get header
    # data <- data[data_rows]
    # data_sub <- lapply(data, function(x){
    #   sub <- x %>% str_split(' +')
    #   sub <- unlist(sub)
    #   sub
    # })
    # full_values = lapply(data_sub,length)
    # data <- do.call("rbind", data_sub[full_values==3])
    # colnames(data) <- header
    data <- data %>% as_tibble()
  original_data <- data
  return(original_data)
}

#' @title argentina
#' @name argentina
#'
#' @description Retrieve Argentine gauge.
#'
#' @param site Argentine gauge number.
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' df <- argentina("29", variable = "discharge")
#' plot(df$Date, df$Q, type = "l")
#' }
#' @export
argentina <- function(site,
                      variable = "discharge",
                      start_date = NULL,
                      end_date = NULL,
                      sites = FALSE,
                      ...) {

  if (sites) {
    return(argentina_sites)
  }

  var_id <- .ina_var_id(variable)
  column_name <- if (variable == "discharge") "Q" else "H"

  start_date <- if (is.null(start_date)) as.Date("1900-01-01") else as.Date(start_date)
  end_date   <- if (is.null(end_date)) Sys.Date() else as.Date(end_date)

  original_data <- try(
    download_argentina_data(site, var_id, start_date, end_date),
    silent = TRUE
  )
  if (is.error(original_data) == TRUE) {
    stop(
      "This gauge does not have a record associated with it and/or the agency website is down."
    )
  }
  if (nrow(original_data$dt) == 0) {
    stop(
      "This gauge does not have a record associated with it and/or the agency website is down."
    )
  }

  dt <- original_data$dt
  reported_unit <- original_data$unit

  if (variable == "stage" && !is.null(reported_unit)) {
    if (grepl("cent[i|\u00ed]metro|^cm$", reported_unit, ignore.case = TRUE)) {
      dt[[column_name]] <- dt[[column_name]] / 100
    } else if (!grepl("^m$|metro", reported_unit, ignore.case = TRUE)) {
      warning(
        "Stage unit reported by the API was '", reported_unit, "', which ",
        "isn't recognized as meters or centimeters -- values may not be ",
        "in meters. Check attr(result, \"unit\") before trusting this column."
      )
    }
  } else if (variable == "discharge" && !is.null(reported_unit) &&
             !grepl("m.?3.?/?s", reported_unit, ignore.case = TRUE)) {
    warning(
      "Discharge unit reported by the API was '", reported_unit, "', not ",
      "the expected m^3/s -- values may not be in m3/s."
    )
  }

  dt <- dt[dt[["Date"]] >= start_date & dt[["Date"]] <= end_date]
  if ("Measurement_type" %in% names(dt)) {
    data.table::setorderv(dt, c("Date", "Measurement_type"))
  } else {
    data.table::setorderv(dt, "Date")
  }

  out <- new_tibble(
    tibble::as_tibble(dt),
    original = tibble::as_tibble(original_data$raw),
    class = "rr_tbl"
  )
  attr(out, "unit") <- reported_unit
  attr(out, "procedimiento") <- original_data$procedimiento
  return(out)
}

download_argentina_data <- function(site, var_id, start_date, end_date) {
  headers <- httr::add_headers(`Accept` = "application/json")

  series_url <- paste0(
    "https://alerta.ina.gob.ar/a5/obs/puntual/series",
    "?estacion_id=", site,
    "&var_id=", var_id,
    "&format=json"
  )
  series_resp <- httr::RETRY("GET", series_url, headers, times = 3, quiet = TRUE)
  if (httr::status_code(series_resp) == 404) {
    stop(
      "404 on the series lookup endpoint -- the inferred path ",
      "'/a5/obs/puntual/series' is likely wrong. Check ",
      "https://alerta.ina.gob.ar/a5/apiUI directly for the real path."
    )
  }
  httr::stop_for_status(series_resp, task = "look up series for this station")
  parsed_series <- jsonlite::fromJSON(httr::content(series_resp, as = "text", encoding = "UTF-8"))
  series_list <- parsed_series$rows
  if (is.null(series_list) || length(series_list) == 0 || nrow(series_list) == 0) {
    stop("No series found for this station + variable combination.")
  }

  proc_id <- series_list$procedimiento$id
  keep <- proc_id %in% c(1, 2)
  if (!any(keep)) {
    stop(
      "This station only has a simulated ('Simulado') series for this ",
      "variable -- no direct measurement or rating curve series available, ",
      "and simulated data is excluded by design."
    )
  }
  series_list <- series_list[keep, ]

  all_series_ids <- series_list$id
  all_procedimientos <- series_list$procedimiento$nombre
  unit_abrev <- series_list$unidades$abrev[1]

  ref_start <- format(start_date, "%Y-%m-%dT00:00:00.000Z")
  ref_end   <- format(end_date, "%Y-%m-%dT23:59:59.000Z")
  value_col <- if (var_id == 40) "Q" else "H"

  raw_list <- vector("list", length(all_series_ids))
  for (i in seq_along(all_series_ids)) {
    obs_url <- paste0(
      "https://alerta.ina.gob.ar/a5/obs/puntual/observaciones",
      "?tipo=puntual",
      "&series_id=", all_series_ids[i],
      "&timestart=", ref_start,
      "&timeend=", ref_end,
      "&format=json"
    )
    obs_resp <- httr::RETRY("GET", obs_url, headers, times = 3, quiet = TRUE)
    if (httr::status_code(obs_resp) == 400) {
      stop(
        "400 Bad Request from the observations endpoint. Server said: ",
        httr::content(obs_resp, as = "text", encoding = "UTF-8")
      )
    }
    if (httr::status_code(obs_resp) == 404) {
      stop(
        "404 on the observations endpoint -- the inferred path ",
        "'/a5/obs/puntual/observaciones' is likely wrong. Check ",
        "https://alerta.ina.gob.ar/a5/apiUI directly for the real path."
      )
    }
    httr::stop_for_status(obs_resp, task = "fetch observations from INA alerta5DBIO")
    obs_i <- jsonlite::fromJSON(httr::content(obs_resp, as = "text", encoding = "UTF-8"))
    if (length(obs_i) > 0 && nrow(obs_i) > 0) {
      obs_i <- data.table::as.data.table(obs_i)
      obs_i[["procedimiento"]] <- all_procedimientos[i]
      obs_i[["series_id"]] <- all_series_ids[i]
      raw_list[[i]] <- obs_i
    }
  }
  raw <- data.table::rbindlist(raw_list, use.names = TRUE, fill = TRUE)

  if (nrow(raw) == 0) {
    return(list(
      dt = data.table::data.table(),
      raw = raw,
      unit = unit_abrev,
      procedimiento = all_procedimientos
    ))
  }

  dt <- data.table::copy(raw)
  dt[["Date"]] <- as.Date(dt[["timestart"]])
  data.table::setnames(dt, "valor", value_col)

  types_present <- unique(dt[["procedimiento"]])
  if (length(types_present) <= 1) {
    cols <- c("Date", value_col)
    dt <- dt[, cols, with = FALSE]
    data.table::setorderv(dt, "Date")
  } else {
    data.table::setnames(dt, "procedimiento", "Measurement_type")
    cols <- c("Date", value_col, "Measurement_type")
    dt <- dt[, cols, with = FALSE]
    data.table::setorderv(dt, c("Date", "Measurement_type"))
  }

  list(dt = dt, raw = raw, unit = unit_abrev, procedimiento = all_procedimientos)
}

.ina_var_id <- function(variable) {
  if (variable == "discharge") return(40L)  # Caudal medio diario -- confirmed
  if (variable == "stage") return(39L)      # Altura hidrometrica media diaria -- confirmed
  stop("variable must be either 'discharge' or 'stage'")
}

#' @title canada
#' @name canada
#'
#' @description Retrieve Canadian gauge data
#'
#' @param site Canadian gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' #Note this no longer relies on tidyhydat but rather retrieves data from an API.
#' df = canada("01AD003")
#' plot(df$Date, df$Q, type='l')
#' }
#' @export

canada <- function(site,
                   variable = "discharge",
                   start_date = NULL,
                   end_date = NULL,
                   sites = FALSE,
                   ...) {
  if (sites) {
    return(canada_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  original_data = try(canada_pull(site,variable,start_date,end_date))
  if (is.error(original_data)) return(stop('This gauge does not have a record associated with it and/or the agency website is down.'))
  original_data <- as_tibble(original_data)
  value_name=colnames(original_data)[grep('Value',colnames(original_data))]
  data <- original_data %>%
    dplyr::select(all_of(c("Date", value_name))) %>%
    rename(!!column_name := value_name) %>%
    filter(.data$Date >= start_date & .data$Date <= end_date)
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

canada_pull=function(site,variable,start_date,end_date){
  param=ifelse(variable=='stage','level','flow')
  url=paste0('https://wateroffice.ec.gc.ca/services/daily_data/csv/inline?stations[]=',site,'&parameters[]=',param,'&start_date=',start_date,'&end_date=',end_date)
  data=content(GET(url))
}

#' @title japan
#' @name japan
#'
#' @description Retrieve Japanese gauge data
#'
#' @param site Japanese gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' start_date <- as.Date("1968-01-01")
#' end_date <- as.Date("1970-12-31")
#' df <- japan("301011281104090", "discharge", start_date, end_date)
#' plot(df$Date, df$Q, type='l')
#' }
#' @importFrom stats na.omit
#' @export
japan <- function(site,
                  variable = "discharge",
                  start_date = NULL,
                  end_date = NULL,
                  sites = FALSE,
                  ...) {

  if (sites) {
    return(japan_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  data <- try(download_japan_data(site, variable, start_date, end_date),silent=TRUE)
  if(is.error(data)==TRUE){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  if(nrow(data)==0){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  return(data)
}

download_japan_data <- function(site, variable, start_date, end_date) {
  base_url <- "http://www1.river.go.jp/cgi-bin/DspWaterData.exe"
  ## Divide timeseries into years, because we can only
  ## scrape data one year at a time.
  start_date <- as.Date(start_date)
  end_date <- as.Date(end_date)

  ts = seq(year(start_date),
                year(end_date))

  ## What should be downloaded?
  if (variable == "stage") {
    kind <- 3
  } else if (variable == "discharge") {
    kind <- 7
  }
  data_list <- list()
  header <- NULL
  output_list=list()
  for (k in 1:length(ts)){
    #day <- dates[k]
    website <- paste0(
      base_url,
      "?KIND=", kind,
      "&ID=", site,
      "&BGNDATE=", paste0(ts[k],'0101'),
      "&ENDDATE=", paste0(ts[k],'1231')
    )
    file <- read_html(website,encoding='ISO-8859-1') %>%
      html_element("body") %>%
      html_table()
    if (is.null(header)) {
      header <- file[1:4, ]
    }
    ## Remove header
    data <- file[5:nrow(file), ]
    data <- rbind(header, data)
    ## Remove header
    data <- data[5:nrow(data), ]
    colnames(data)=c('Month',seq(1:31))
    data$Month = gsub("[^0-9.]", "", data$Month)
    data= as_tibble(data)

    tab=list()
    for(i in 1:nrow(data)){
      mnth=data$Month[i]
      days=colnames(data)[2:ncol(data)]
      values=suppressWarnings(as.numeric(data[i,2:ncol(data)]))
      year = ts[k]
      dates=as.Date(paste(year,mnth,days,sep='-'),format='%Y-%m-%d')
      tab[[i]] = data.frame(dates,values)
    }
    combined = do.call("rbind", tab)
    combined = na.omit(combined)
    output_list[[k]]=combined
  }
  combined_output=do.call('rbind',output_list)
  colnames(combined_output) = c('Date',ifelse(variable=='discharge','Q','H'))
  combined_output = new_tibble(combined_output,
                      original=data,
                      class = "rr_tbl")
  combined_output = combined_output[order(combined_output$Date),]
  return(combined_output)
}

#' @title quebec
#' @name quebec
#'
#' @description Retrieve Quebec gauge data
#'
#' @param site quebec gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \donttest{
#' site <- "074903"
#' x <- quebec(site, variable = "stage")
#' plot(x$Date, x$H, type='l')
#' }
#' @importFrom data.table fread
#' @export
quebec <- function(site,
                   variable,
                   start_date = NULL,
                   end_date = NULL,
                   sites = FALSE,
                   ...) {
  if (sites) {
    return(quebec_sites)
  }
  f <- as.character(site)
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  p_code=ifelse(variable=='discharge','Q','N')
  target_url <- paste0('https://www.cehq.gouv.qc.ca/depot/historique_donnees/fichier/', f,'_',p_code,'.txt')

  out=tempfile()

  download_status <- tryCatch({
    utils::download.file(url = target_url, destfile = out, silent = TRUE)
    TRUE
  }, error = function(e) {
    FALSE
  })

  if (!download_status) return(stop('This gauge does not have a record associated with it and/or the agency website is down.'))

  raw_data <- fread(out, fill = TRUE, colClasses = "character",encoding='Latin-1')
  # Find the header row dynamically
  header_idx <- grep('Date', raw_data$V2)

  if (length(header_idx) == 0) {
    return(stop('This gauge does not have a record associated with it and/or the agency website is down.'))
  }

  # Slicing from the row after the header and selecting columns 2 and 3
  original_data <- raw_data[(header_idx + 1):nrow(raw_data),1:4]
  original_data = as_tibble(original_data)
  colnames(original_data) = unlist(raw_data[header_idx,1:4])

  data = as.data.frame(original_data[,2:3])
  data[,1] = as.Date(data[,1],format='%Y/%m/%d')
  data[,2] = as.numeric(data[,2])
  colnames(data)=c('Date',ifelse(p_code=='Q','Q','H'))
  data = data[data$Date>=start_date&data$Date<=end_date,]

  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

.get_start_date <- function(start_date) {
  ## Format start date
  if (is.null(start_date)) {
    start_date <- "1900-01-01"
  }
  return(start_date)
}

.get_end_date <- function(end_date) {
  ## Format end date - if not specified then use current date
  if (is.null(end_date))
    end_date <- Sys.time() %>%
      as.Date() %>%
      format("%Y-%m-%d")
  return(end_date)
}

.get_column_name <- function(variable) {
  ## Get appropriate column name for return object
  if (variable == "stage") {
    colnm <- "H"
  } else if (variable == "discharge") {
    colnm <- "Q"
  } else {
    stop(sprintf("Variable %s is not available", variable))
  }
  return(colnm)
}

## .build_rr_tbl <- function(x, variable, original_data, ...) {
##   if (variable == "discharge") {
##     units <- "m3/s"
##   } else if (variable == "stage") {
##     units <- "m"
##   }
##   out <- new_tibble(
##     data,
##     variable = variable,
##     original = original_data,
##     class = c("rr_tbl")
##   )
## }

#' @title ireland
#' @name ireland
#'
#' @description Retrieve Ireland gauge data
#'
#' @param site Ireland gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' df <- ireland('25014',"discharge")
#' plot(df$Date, df$Q)
#' }
#' @export

ireland <- function(site,
                    variable,
                    start_date = NULL,
                    end_date = NULL,
                    sites = FALSE,
                    ...) {
  if (sites) {
    return(ireland_sites)
  }

  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)

  original_data <- try(download_ireland_data(
    site, variable, start_date, end_date
  ), silent = TRUE)

  if(is.error(original_data) == TRUE | length(original_data) == 0) {
    stop('This gauge does not have a record associated with it and/or the agency website is down.')
  }

  data <- original_data
  column_name = ifelse(variable == "discharge", "Q", "H")

  data <- data %>% dplyr::select(all_of(c("Timestamp", "Value"))) %>%
    rename(Date = "Timestamp", !!column_name := "Value")

  data$Date = as.Date(data$Date)
  data[, 2] = as.numeric(data[, 2])

  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )

  return(out)
}

download_ireland_data = function(site, variable, start_date, end_date) {
  if (variable == "stage") {
    path = paste0('https://waterlevel.ie/hydro-data/data/internet/stations/0/', site, '/S/year.json')
  } else if (variable == "discharge") {
    path = paste0('https://waterlevel.ie/hydro-data/data/internet/stations/0/', site, '/Q/year.json')
  }

  df = fromJSON(path)
  data = as.data.frame(do.call('rbind', df$data))
  colnames(data) = unlist(str_split(df$columns[1], ','))
  data = data[as.Date(data$Timestamp) <= end_date & as.Date(data$Timestamp) >= start_date, ]
  return(data)
}

#' @title scotland
#' @name scotland
#'
#' @description Retrieve Scotland gauge data
#'
#' @param site Scotland gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' df <- scotland("14969", "discharge")
#' plot(df$Date, df$Q, type='l')
#' }
#' @export
scotland <- function(site,
                  variable = "discharge",
                  start_date = NULL,
                  end_date = NULL,
                  sites = FALSE,
                  ...) {

  if (sites) {
    return(scotland_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  data <- try(download_scotland_data(site, variable, start_date, end_date),silent=TRUE)
  if(is.error(data)==TRUE){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  if(nrow(data)==0){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  return(data)
}
# Discharge = Q, Level = SG
download_scotland_data = function(site, variable, start_date = NULL, end_date = NULL) {
  param = ifelse(variable == 'stage', 'SG', 'Q')
  param_output = .get_column_name(variable)
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)

  website = paste0('https://timeseries.sepa.org.uk/KiWIS/KiWIS?service=kisters&type=queryServices&datasource=0&request=gettimeseriesvalues&ts_path=1/',
                   site, '/', param, '/HDay.Mean&from=', start_date, '&to=', end_date, '&metadata=true&returnfields=Timestamp,Value,Quality%20Code')

  data_pull = read_html(website) %>% html_table()
  data = data_pull[[1]]
  index = grep('Timestamp', data$X1)
  headers = data[index, ]
  df = as.data.frame(data[(index + 1):nrow(data), ])
  colnames(df) = headers
  df = df[!is.na(df$Timestamp)&df$Timestamp!='',]
  original_data = new_tibble(df)

  modified_data <- df %>% dplyr::select(all_of(c("Timestamp", "Value"))) %>%
    rename(Date = "Timestamp", !!param_output := "Value")
  modified_data$Date = as.Date(modified_data$Date)
  modified_data[, 2] = as.numeric(modified_data[, 2])

  out <- new_tibble(
    modified_data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

#' @title australia
#' @name australia
#'
#' @description Retrieve Australian gauge data
#'
#' @param site Australian gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' sites <- australia(sites = TRUE)
#' df <- australia(sites$site[1], "stage")
#' plot(df$Date, df$H, type='l')
#' }
#' @importFrom stats runif
#' @export
australia <- function(site,
                      variable = "discharge",
                      start_date = NULL,
                      end_date = NULL,
                      sites = FALSE,
                      ...) {

  if (sites) {
    return(australia_sites)
  }
  Sys.sleep(runif(1,1,3))
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)

  ## Recode variables from RivRetrieve convention to BoM convention
  if (variable == "stage") {
    bom_variable <- "Water Course Level"
  } else if (variable == "discharge") {
    bom_variable <- "Water Course Discharge"
  }

  ## Retrieve 'timeseries_id' using the bom package.
  id = get_timeseries_id(bom_variable, site,'DMQaQc.Merged.DailyMean.24HR')

  ##Interface with the BoM website.
  web = 'http://www.bom.gov.au/waterdata/services?service=kisters&type=queryServices&request=getTimeseriesValues&datasource=0&format=csv&ts_id='
  website = paste0(web, id$ts_id, '&from=', start_date, 'T00:00:00.000&to=', end_date,'T00:00:00.000&returnfields=Timestamp,Value,Quality%20Code,Interpolation%20Type&language=en&downloadaszip=false&timezone=individual&csvdiv=%2C&md_returnfields=station_longname,station_no,station_latitude,station_longitude,parametertype_name,ts_name,ts_unitname,custom_attributes&custattr_returnfields=DATA_OWNER_NAME&metadata=true&downloadfilename=csv.australia_file')
  headers <- add_headers(`User-Agent` = "Mozilla/5.0 (Windows NT 10.0; Win64; x64)")
  response <- GET(website, headers)
  out = content(response)
  #out = tempfile()
  #test = download.file(website, out, method='curl', quiet=TRUE)
  original_data = read_delim(out,skip=9)
  if(is.error(original_data)==TRUE){stop('This gauge does not have a record associated with it and/or the agency website is down.')}

  ## Create return object
  original_data <- as_tibble(original_data)
  data <- original_data %>%
    mutate(Date = as.Date(.data$'#Timestamp')) %>%
    dplyr::select(c("Date", "Value")) %>%
    rename(!!column_name := "Value")
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

#' @title Query the BoM WISKI API
#' @description
#' This function queries the Bureau of Meteorology Water Data KISTERS API.
#' A parameter list is passed to make request and the JSON return is parsed
#' depending on what is requested. This function can be used if you want to
#' build your own JSON queries.
#' @param params A named list of parameters.
#' @return
#' A tibble is returned with the columns depending on the request. For
#' \code{get_timeseries} requests, a tibble with zero rows is returned
#' if there is no data available for that query.
make_bom_request <- function(params) {
  bom_url <- "http://www.bom.gov.au/waterdata/services"

  base_params <- list(
    "service" = "kisters",
    "type" = "QueryServices",
    "format" = "json"
  )

  r <- tryCatch(
    {
      r <- httr::RETRY("GET", bom_url, query = c(base_params, params), times = 5, quiet = TRUE)
      httr::stop_for_status(r, task = "request water data from BoM")
      httr::warn_for_status(r, task = "request water data from BoM")
    },
    error = function(e) {
      message(strwrap(
        prefix = " ", initial = "",
        "Request for water data failed. Check your request and make sure
         http://www.bom.gov.au/waterdata/ is online"
      ))
      message("Error message:")
      message(e$message)
    },
    warning = function(w) {
      message("Request for water data raised a warning. Warning message:")
      message(w$message)
    }
  )
  json <- jsonlite::fromJSON(httr::content(r, "text"))

  if (params$request %in% c(
    "getParameterList",
    "getSiteList",
    "getStationList",
    "getTimeseriesList"
  )) {
    if (json[1] == "No matches.") {
      stop("No parameter type and station number match found")
    }
    colnames(json) <- json[1, ]
    tbl <- dplyr::slice(tibble::as_tibble(json), -1)
  } else if (params$request == "getTimeseriesValues") {
    column_names <- unlist(stringr::str_split(json$columns, ","))
    if (length(json$data[[1]]) == 0) {
      tbl <- tibble::tibble(
        Timestamp = lubridate::as_datetime(lubridate::ymd()),
        Value = numeric(),
        `Quality Code` = integer()
      )
    } else {
      colnames(json$data[[1]]) <- column_names
      tbl <- tibble::as_tibble(json$data[[1]])
    }
  }
  return(tbl)
}

#' @title Retrieve the timeseries ID
#' @description
#' `get_timeseries_id` retrieves the timeseries ID that can be used to obtain
#' values for a parameter type, station and timeseries combination.
#' @param parameter_type The parameter of interest (e.g. Water Course
#' Discharge).
#' @param station_number The AWRC station number.
#' @param ts_name The BoM time series name (e.g. DMQaQc.Merged.DailyMean.24HR).
#' @return
#' Returns a tibble with columns station_name, station_no, station_id, ts_id,
#' ts_name, parametertype_id, parametertype_name.
get_timeseries_id <- function(parameter_type, station_number, ts_name) {
  params <- list(
    "request" = "getTimeseriesList",
    "parametertype_name" = parameter_type,
    "ts_name" = ts_name,
    "station_no" = station_number
  )

  get_bom_request <- make_bom_request(params)

  return(get_bom_request)
}

#' @title uk
#' @name uk
#'
#' @description Retrieve UK gauge data
#'
#' @param site UK gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \donttest{
#' site <- "http://environment.data.gov.uk/hydrology/id/stations/3c5cba29-2321-4289-a1fd-c355e135f4cb"
#' x <- uk(site, variable = "discharge")
#' plot(x$Date, x$Q, type='l')
#' }
#' @export
uk <- function(site,
               variable,
               start_date = NULL,
               end_date = NULL,
               sites = FALSE,
               ...) {

  if (sites) {
    return(uk_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  original_data <- try(download_uk_data(
    site, variable, start_date, end_date
  ))
  if(is.error(original_data)==TRUE|length(original_data)==0){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  data <- original_data
  ## Aggregate to get daily data
  if (variable == "stage") {
    data <- data %>%
      group_by(date) %>%
      summarize(value = sum(.data$value), count = n()) %>%
      filter(count == 96)
  }
  ## Merge with complete time series in case any missing,
  ## then select columns
  complete_ts <- seq(min(data$date), max(data$date), by = "1 day")
  data <- tibble(date = complete_ts) %>%
    left_join(data, by = "date") %>%
    dplyr::select(all_of(c("date", "value"))) %>%
    rename(Date = "date", !!column_name := "value")
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

download_uk_data <- function(site, variable, start_date, end_date) {
  if (variable == "stage") {
    ptn <- "level-i-900-m-qualified"
  } else if (variable == "discharge") {
    ptn <- "flow-m-86400-m3s-qualified"
  }
  ## Parse site code and check whether site has data for given variable
  first <- strsplit(site, "/")
  site <- first[[1]][[7]]
  base_url <- "http://environment.data.gov.uk"
  url <- paste0(
    base_url,
    "/hydrology/id/measures?station=", site
  )
  r <- GET(url)
  c <- content(r)
  measures <- sapply(c$items, FUN = function(x) x$notation)
  ix <- grep(ptn, measures)
  if (length(ix) != 1) {
    stop(sprintf("Site %s does not have %s data", site, variable))
  } else {
    notation <- measures[ix]
  }
  ## Get data
  done <- FALSE
  data_list <- list()
  ## API places a hard limit of 2000000 records which is fine for
  ## daily discharge but has to be managed for 15-min stage data.
  while (!done) {
    url <- paste0(
      base_url,
      "/hydrology/id/measures/", notation, "/readings",
      "?mineq-date=", start_date, "&maxeq-date=", end_date,
      "&_limit=2000000"
    )
    r <- GET(url)
    c <- content(r)
    selected_items <- lapply(
      c$items, FUN = function(x) x[c("date", "dateTime", "value", "quality")]
    )
    x <- list.stack(selected_items) %>%
      as_tibble() %>%
      mutate(date = as.Date(.data$date))
    max_date <- max(x$date)
    if ((nrow(x) == 2000000) && (max_date < end_date)) {
      start_date <- max_date
    } else {
      done <- TRUE
    }
    data_list[[length(data_list) + 1]] <- x
  }
  original_data <- do.call("rbind", data_list)
  return(original_data)
}

#' @title sweden
#' @name sweden
#'
#' @description Retrieve Swedish gauge data
#'
#' @param site sweden gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \donttest{
#' df = sweden(site='2000',variable='discharge')
#' plot(df$Date,df$Q,type='l')
#' }
#' @importFrom data.table fread
#' @export
sweden <- function(site,
                   variable,
                   start_date = NULL,
                   end_date = NULL,
                   sites = FALSE,
                   ...) {

  if (sites) {
    return(sweden_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  original_data <- try(download_sweden_data(
    site, variable, start_date, end_date
  ),silent=TRUE)
  if(is.error(original_data)==TRUE|length(original_data)==0){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  data <- original_data

  data <- data%>%dplyr::select(all_of(c("date", "value"))) %>%
    rename(Date = "date", !!column_name := "value")
  if(variable=='stage'){
    data$H = data$H*.01
  }
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

download_sweden_data = function(site, variable, start_date, end_date){
  if (variable == "stage") {
    site=paste0('https://opendata-download-hydroobs.smhi.se/api/version/1.0/parameter/3/station/',site,'/period/corrected-archive/data.json')
  } else if (variable == "discharge") {
    site=paste0('https://opendata-download-hydroobs.smhi.se/api/version/1.0/parameter/1/station/',site,'/period/corrected-archive/data.json')
  }
  data = fromJSON(site)
  ts = data$value
  ts$date = as.Date(as.POSIXct(ts$date/1000, origin='1970-01-01'))

  original_data <- ts[ts$date>=as.Date(start_date)&ts$date<=as.Date(end_date),]
  return(original_data)
}

#' @title germany
#' @name germany
#'
#' @description Retrieve German gauge data.
#'
#' @param site German gauge id.
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' # Note the German API currently only provides access to stage data from the previous 30 days.
#' site <- "593647aa-9fea-43ec-a7d6-6476a76ae868"
#' stage <- germany(site, variable = "stage")
#' plot(stage$Date, stage$H, type = "l")
#' }
#' @export
germany <- function(site,
                    variable = "discharge",
                    start_date = NULL,
                    end_date = NULL,
                    sites = FALSE,
                    ...) {

  if (sites) {
    return(german_sites)
  }

  if (!variable %in% c("stage", "discharge")) {
    stop("variable must be either 'stage' or 'discharge'")
  }
  param_code  <- if (variable == "stage") "W" else "Q"
  column_name <- if (variable == "discharge") "Q" else "H"
  latest_available <- Sys.Date()
  end_date <- if (is.null(end_date)) Sys.Date() else as.Date(end_date)

  # --- PEGELONLINE-specific: clamp start_date to the 30-day window --------
  earliest_available <- Sys.Date() - 30
  start_date <- if (is.null(start_date)) earliest_available else as.Date(start_date)
  if (start_date < earliest_available) {
    warning(
      "PEGELONLINE only retains 30 days of raw data. Requested start_date ",
      start_date, " is earlier than that -- clamping to ", earliest_available, "."
    )
    start_date <- earliest_available
  }

  if (end_date < earliest_available || end_date > latest_available) {
    warning(
      "PEGELONLINE only retains 30 days of raw data. Requested end_date ",
      end_date, " is outside of data range -- clamping to ", latest_available, "."
    )
    end_date <- latest_available
  }

  original_data <- try(
    download_germany_data(site, param_code, start_date, end_date),
    silent = TRUE
  )
  if (is.error(original_data) == TRUE || nrow(original_data) == 0) {
    stop(
      "This gauge does not have a record associated with it and/or the ",
      "agency website is down."
    )
  }

  dt <- data.table::as.data.table(original_data)
  dt[["Date"]] <- as.Date(dt[["timestamp"]])
  data.table::setnames(dt, "value", column_name)

  # Subset and aggregate daily mean without bare column references or `.`
  keep_rows <- dt[["Date"]] >= start_date & dt[["Date"]] <= end_date
  dt <- dt[keep_rows, list(value = mean(get(column_name), na.rm = TRUE)), by = "Date"]

  data.table::setnames(dt, "value", column_name)
  data.table::setorderv(dt, "Date")

  # PEGELONLINE reports stage (W) in centimeters relative to a local gauge
  # datum, converting to meters.
  if (variable == "stage") {
    dt[[column_name]] <- dt[[column_name]] / 100
  }

  out <- new_tibble(
    tibble::as_tibble(dt),
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

download_germany_data <- function(site, param_code, start_date, end_date) {
  url <- paste0(
    "https://www.pegelonline.wsv.de/webservices/rest-api/v2/stations/",
    site, "/", param_code, "/measurements.json",
    "?start=", format(start_date, "%Y-%m-%d"),
    "&end=", format(end_date, "%Y-%m-%d")
  )
  resp <- httr::RETRY("GET", url, times = 3, quiet = TRUE)
  httr::stop_for_status(resp, task = "request measurements from PEGELONLINE")
  parsed <- jsonlite::fromJSON(httr::content(resp, as = "text", encoding = "UTF-8"))
  if (length(parsed) == 0) {
    return(data.table::data.table(timestamp = character(), value = numeric()))
  }
  data.table::as.data.table(parsed)
}

#' @title brazil
#' @name brazil
#'
#' @description Retrieve Brazilian gauge data
#'
#' @param site Brazilian gauge number
#' @param variable Character. Either `stage` or `discharge`.
#' @param start_date Character. Optional start date with format
#'   YYYY-MM-DD. Default is 1900-01-01.
#' @param end_date Character. End date with format YYYY-MM-DD.
#'   Default is the current date.
#' @param sites Logical. If TRUE, returns a list of measurement
#'   sites.
#' @param ... Additional arguments. None implemented.
#'
#' @return data frame of discharge time-series
#' @examples
#' \dontrun{
#' df <- brazil('12650000')
#' plot(df$Date, df$Q, type='l')
#' }
#' @export
brazil <- function(site,
                   variable = "discharge",
                   start_date = NULL,
                   end_date = NULL,
                   sites = FALSE,
                   ...) {

  if (sites) {
    return(brazil_sites)
  }
  start_date <- .get_start_date(start_date)
  end_date <- .get_end_date(end_date)
  column_name <- .get_column_name(variable)
  original_data <- try(download_hidroweb_data(site, variable),silent=TRUE)
  if(is.error(original_data)==TRUE){stop('This gauge does not have a record associated with it and/or the agency website is down.')}
  data <- parse_hidroweb_data(
    original_data, variable = variable
  )
  data <- data %>%
    dplyr::select(all_of(c("date", "Value"))) %>%
    rename(Date = "date") %>%
    rename(!!column_name := "Value") %>%
    filter(.data$Date >= start_date & .data$Date <= end_date)
  out <- new_tibble(
    data,
    original = original_data,
    class = "rr_tbl"
  )
  return(out)
}

download_hidroweb_data <- function(site, variable, ...) {
  ## Download data to a temporary location
  base_url <- "https://www.snirh.gov.br/hidroweb/rest/api/documento/convencionais?tipo=3&documentos="
  out <- tempfile()
  ## res <- download.file(
  ##   paste0(base_url, site), out,
  ##   method = "curl", quiet = TRUE
  ## )
  res <- GET(paste0(base_url, site), write_disk(out, overwrite = TRUE))
  if (res$status != 200) {
    stop(paste0("Could not download data for requested site (HTTP status: ", res$status, ")"))
  }
  tmpdir <- tempdir()
  a <- try(unzip(out, exdir = tmpdir), silent = TRUE)
  if (variable == "discharge") {
    f <- unzip(a[grep("^(.*)/vazoes_(.*).zip", a)], exdir = tmpdir)
  } else if (variable == "stage") {
    f <- unzip(a[grep("^(.*)/cotas_(.*).zip", a)], exdir = tmpdir)
  }
  original_data <- read_hidroweb_data(f)
  original_data <- as_tibble(original_data)
  return(original_data)
}

read_hidroweb_data <- function(filename, ...) {
  ## TODO unsure how reliable 'skip=13' will be
  data <- suppressWarnings(
    read_delim(
      filename, delim = ";",
      locale = locale(decimal_mark = ","),
      skip = 13,
      progress = FALSE, show_col_types = FALSE)
  )
  data
}

parse_hidroweb_data <- function(data, variable = "stage", ...) {
  id_cols <- c("EstacaoCodigo", "NivelConsistencia", "Data", "Hora")
  if (variable == "stage") {
    prefix <- "Cota"
  } else if (variable == "discharge") {
    prefix <- "Vazao"
    id_cols <- c(id_cols, "MetodoObtencaoVazoes")
  }
  ## Pivot longer, while keeping values and status flags
  data <- data %>%
    dplyr::select(all_of(id_cols), starts_with(prefix)) %>%
    mutate(across(starts_with(prefix), as.character)) %>%
    rename_with(
      .cols = ends_with("Status"),
      .fn = str_replace, pattern = "Status",
      replace = "_Status"
    ) %>%
    rename_with(
      .cols = matches(paste0(prefix, "[0-9]+$")),
      .fn = str_c,
      "_Value"
    )
  data <- data %>%
    pivot_longer(
      -all_of(id_cols),
      names_to = c("day", ".value"),
      names_sep = "_"
    )

  ## Convert strings to numeric
  data <- data %>%
    mutate(
      Value = gsub(",", "", .data$Value),
      Value = as.numeric(.data$Value),
      Status = gsub(";", "", .data$Status),
      Status = na_if(.data$Status, ""),
      Status = as.numeric(.data$Status)
    )


  ## Convert units if stage
  if (variable == "stage") {
    data <- data %>% mutate(Value = Value / 100.)
  }



  ## Get time series
  data <- data %>%
    mutate(Data = as.Date(.data$Data, format = "%d/%m/%Y")) %>%
    mutate(
      year = lubridate::year(.data$Data),
      month = lubridate::month(.data$Data),
      day = gsub(prefix, "", .data$day),
      day = as.numeric(.data$day)
    )

  ## Clean `Hora` column
  data <- data %>%
    mutate(
      Hora = gsub("^(.*) ([0-9]{2}):([0-9]{2}):([0-9]{2})$", "\\2", .data$Hora),
      Hora = as.numeric(.data$Hora)
    )

  ## Note that make_date will give NA for illegitimate
  ## dates (e.g. 31 Feb), so we can filter by NA
  data <- data %>%
    mutate(date = lubridate::make_date(.data$year, .data$month, .data$day)) %>%
    filter(!is.na(.data$date)) %>%
    dplyr::select(-all_of(c("Data", "year", "month", "day"))) %>%
    arrange(.data$date)

  ## Lastly we ensure our object is a complete time series
  ## without any gaps
  complete_ts <-
    tibble(date = seq.Date(data$date[1], rev(data$date)[1], by = "1 day"))
  data <- data %>%
    full_join(complete_ts, by = "date")
  data
}

#' RivRetrieve: Retrieve global river and stage data
#'
#' @aliases NULL RivRetrieve-package
#' @import dataRetrieval
#' @import devtools
#' @import dplyr
#' @import httr
#' @import jsonlite
#' @import lubridate
#' @import RSelenium
#' @import rvest
#' @import stringr
#' @import tibble
#' @import tidyhydat
#' @import tidyr
#' @importFrom BBmisc is.error
#' @importFrom readr read_delim locale
#' @importFrom rlist list.select list.stack
#' @importFrom rlang .data :=
#' @importFrom stats setNames time
#' @importFrom utils download.file unzip
#' @importFrom utils read.csv
#'
#' @examples
#' \dontrun{
#' print("TODO")
#' }
"_PACKAGE"

---
title: "Introduction to the RivRetrieve R package"
author: 'Ryan M. Riggs and Simon Moulds'
date: "`r format(Sys.time(), '%B %d, %Y')`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{my-vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding[utf8]{inputenc}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",warning=FALSE,message=FALSE,fig.width=7
)
```

## Here is a quick tutorial for RivRetrieve functions:

The `RivRetrieve` package is developed to efficiently access and download global river gauge data into an R environment. While numerous river data exists, our package is currently limited to daily measurements of river stage (meters) and discharge (cubic meters per second) for: Argentina, Australia, Brazil, Canada, Chile, France, Germany, Ireland, Japan, Quebec, Scotland, South Africa, the United Kingdom, and the United States.

```{r setup}
library(RivRetrieve)
# Amazon River near Obidos, Brazil
## siteNumber='17050001'
## discharge=brazil(site=siteNumber,variable='discharge')
## stage=brazil(site=siteNumber,variable='stage')
## plot(discharge$Date,discharge$Q, type='l',xlab='',ylab='Discharge (cms)')

# Mississippi River near Baton Rouge, LA, USA
siteNumber="07374000"
discharge=usa(site=siteNumber,variable='discharge')
stage=usa(site=siteNumber,variable='stage')
plot(discharge$Date,discharge$Q, type='l',xlab='',ylab='Discharge (cms)')



```

RivRetrieve automatically outputs a dataframe containing a `Date` column of `datetime` values and either a `H` or `Q` `numeric` column for the stage and discharge variable, respectively.

Stage is provided in meters and discharge is provided in cubic meters per second.

## Accessing the raw gauge data:

```{r,echo=TRUE}
library(knitr)
## kable(raw[1:5,],caption='Raw Brazilian gauge data')

raw=original(discharge)
kable(raw[1:5,],caption='Raw US gauge data')
```

## Site locations for all agencies can also be found:

```{r,echo=TRUE}
## brazilSites=brazil(sites=TRUE)
## kable(brazilSites[1:10,],caption='Example Brazilian river gauge locations')

usaSites=usa(sites=TRUE)
kable(usaSites[1:10,],caption='Example US river gauge locations')
```

## Date defined retrieval

Specific time periods can also be accessed if the entire time series is not needed.

```{r}
# Annual timeseries
## siteNumber='17050001'
## recent=brazil(site=siteNumber,variable='stage',start_date = start,end_date = end)
## plot(recent$Date,recent$H,type='l', xlab='',ylab='Stage (m)')

siteNumber="07374000"
start='2009-01-01'
end='2010-01-31'
recent=usa(site=siteNumber,variable='stage',start_date = start,end_date = end)
plot(recent$Date,recent$H,type='l', xlab='',ylab='Stage (m)')
```

---
title: "Introduction to the RivRetrieve R package"
author: 'Ryan M. Riggs and Simon Moulds'
date: "`r format(Sys.time(), '%B %d, %Y')`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{my-vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding[utf8]{inputenc}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",warning=FALSE,message=FALSE,fig.width=7
)
```

## Here is a quick tutorial for RivRetrieve functions:

The `RivRetrieve` package is developed to efficiently access and download global river gauge data into an R environment. While numerous river data exists, our package is currently limited to daily measurements of river stage (meters) and discharge (cubic meters per second) for: Australia, Brazil, Canada, Chile, France, Ireland, Japan, Quebec, Scotland, South Africa, the United Kingdom, and the United States.

```{r setup}
library(RivRetrieve)
# Amazon River near Obidos, Brazil
## siteNumber='17050001'
## discharge=brazil(site=siteNumber,variable='discharge')
## stage=brazil(site=siteNumber,variable='stage')
## plot(discharge$Date,discharge$Q, type='l',xlab='',ylab='Discharge (cms)')

# Mississippi River near Baton Rouge, LA, USA
siteNumber="07374000"
discharge=usa(site=siteNumber,variable='discharge')
stage=usa(site=siteNumber,variable='stage')
plot(discharge$Date,discharge$Q, type='l',xlab='',ylab='Discharge (cms)')



```

RivRetrieve automatically outputs a dataframe containing a `Date` column of `datetime` values and either a `H` or `Q` `numeric` column for the stage and discharge variable, respectively.

Stage is provided in meters and discharge is provided in cubic meters per second.

## Accessing the raw gauge data:

```{r,echo=TRUE}
library(knitr)
## kable(raw[1:5,],caption='Raw Brazilian gauge data')

raw=original(discharge)
kable(raw[1:5,],caption='Raw US gauge data')
```

## Site locations for all agencies can also be found:

```{r,echo=TRUE}
## brazilSites=brazil(sites=TRUE)
## kable(brazilSites[1:10,],caption='Example Brazilian river gauge locations')

usaSites=usa(sites=TRUE)
kable(usaSites[1:10,],caption='Example US river gauge locations')
```

## Date defined retrieval

Specific time periods can also be accessed if the entire time series is not needed.

```{r}
# Annual timeseries
## siteNumber='17050001'
## recent=brazil(site=siteNumber,variable='stage',start_date = start,end_date = end)
## plot(recent$Date,recent$H,type='l', xlab='',ylab='Stage (m)')

siteNumber="07374000"
start='2009-01-01'
end='2010-01-31'
recent=usa(site=siteNumber,variable='stage',start_date = start,end_date = end)
plot(recent$Date,recent$H,type='l', xlab='',ylab='Stage (m)')
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/australia.R
\name{make_bom_request}
\alias{make_bom_request}
\title{Query the BoM WISKI API}
\usage{
make_bom_request(params)
}
\arguments{
\item{params}{A named list of parameters.}
}
\value{
A tibble is returned with the columns depending on the request. For
\code{get_timeseries} requests, a tibble with zero rows is returned
if there is no data available for that query.
}
\description{
This function queries the Bureau of Meteorology Water Data KISTERS API.
A parameter list is passed to make request and the JSON return is parsed
depending on what is requested. This function can be used if you want to
build your own JSON queries.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sweden.R
\name{sweden}
\alias{sweden}
\title{sweden}
\usage{
sweden(site, variable, start_date = NULL, end_date = NULL, sites = FALSE, ...)
}
\arguments{
\item{site}{sweden gauge number}

\item{variable}{Character. Either \code{stage} or \code{discharge}.}

\item{start_date}{Character. Optional start date with format
YYYY-MM-DD. Default is 1900-01-01.}

\item{end_date}{Character. End date with format YYYY-MM-DD.
Default is the current date.}

\item{sites}{Logical. If TRUE, returns a list of measurement
sites.}

\item{...}{Additional arguments. None implemented.}
}
\value{
data frame of discharge time-series
}
\description{
Retrieve Swedish gauge data
}
\examples{
\donttest{
df = sweden(site='2000',variable='discharge')
plot(df$Date,df$Q,type='l')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3-methods.R
\name{plot.rr_tbl}
\alias{plot.rr_tbl}
\title{Plot values}
\usage{
\method{plot}{rr_tbl}(x, ...)
}
\arguments{
\item{x}{Tibble.}

\item{...}{Additional arguments. None implemented.}
}
\value{
ggplot2
}
\description{
Plot values
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uk.R
\name{uk}
\alias{uk}
\title{uk}
\usage{
uk(site, variable, start_date = NULL, end_date = NULL, sites = FALSE, ...)
}
\arguments{
\item{site}{UK gauge number}

\item{variable}{Character. Either \code{stage} or \code{discharge}.}

\item{start_date}{Character. Optional start date with format
YYYY-MM-DD. Default is 1900-01-01.}

\item{end_date}{Character. End date with format YYYY-MM-DD.
Default is the current date.}

\item{sites}{Logical. If TRUE, returns a list of measurement
sites.}

\item{...}{Additional arguments. None implemented.}
}
\value{
data frame of discharge time-series
}
\description{
Retrieve UK gauge data
}
\examples{
\donttest{
site <- "http://environment.data.gov.uk/hydrology/id/stations/3c5cba29-2321-4289-a1fd-c355e135f4cb"
x <- uk(site, variable = "discharge")
plot(x$Date, x$Q, type='l')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RivRetrieve-package.R
\docType{package}
\name{RivRetrieve-package}
\alias{RivRetrieve}
\alias{RivRetrieve-package}
\title{RivRetrieve: Retrieve global river and stage data}
\description{
Provides access to global river gauge data from a variety of national-level river agencies. The package interfaces with the national-level agency websites to provide access to river gauge locations, river discharge, and river stage. Currently, the package is available for the following countries: Argentina, Australia, Brazil, Canada, Chile, France, Germany, Japan, South Africa, the United Kingdom, and the United States.
}
\examples{
\dontrun{
print("TODO")
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Ryan-Riggs/RivRetrieve}
}

}
\author{
\strong{Maintainer}: Ryan Riggs \email{ryanriggs7@gmail.com} (\href{https://orcid.org/0000-0001-6834-9469}{ORCID})

Authors:
\itemize{
  \item Simon Moulds \email{sim.moulds@gmail.com} (\href{https://orcid.org/0000-0002-7297-482X}{ORCID})
  \item Michel Wortmann \email{michel.wortmann@ouce.ox.ac.uk} (\href{https://orcid.org/0000-0002-1879-7674}{ORCID})
  \item Louise Slater \email{louise.slater@ouce.ox.ac.uk} (\href{https://orcid.org/0000-0001-9416-488X}{ORCID})
  \item George Allen \email{geoallen@vt.edu} (\href{https://orcid.org/0000-0001-8301-5301}{ORCID})
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/australia.R
\name{get_timeseries_id}
\alias{get_timeseries_id}
\title{Retrieve the timeseries ID}
\usage{
get_timeseries_id(parameter_type, station_number, ts_name)
}
\arguments{
\item{parameter_type}{The parameter of interest (e.g. Water Course
Discharge).}

\item{station_number}{The AWRC station number.}

\item{ts_name}{The BoM time series name (e.g. DMQaQc.Merged.DailyMean.24HR).}
}
\value{
Returns a tibble with columns station_name, station_no, station_id, ts_id,
ts_name, parametertype_id, parametertype_name.
}
\description{
\code{get_timeseries_id} retrieves the timeseries ID that can be used to obtain
values for a parameter type, station and timeseries combination.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quebec.R
\name{quebec}
\alias{quebec}
\title{quebec}
\usage{
quebec(site, variable, start_date = NULL, end_date = NULL, sites = FALSE, ...)
}
\arguments{
\item{site}{quebec gauge number}

\item{variable}{Character. Either \code{stage} or \code{discharge}.}

\item{start_date}{Character. Optional start date with format
YYYY-MM-DD. Default is 1900-01-01.}

\item{end_date}{Character. End date with format YYYY-MM-DD.
Default is the current date.}

\item{sites}{Logical. If TRUE, returns a list of measurement
sites.}

\item{...}{Additional arguments. None implemented.}
}
\value{
data frame of discharge time-series
}
\description{
Retrieve Quebec gauge data
}
\examples{
\donttest{
site <- "074903"
x <- quebec(site, variable = "stage")
plot(x$Date, x$H, type='l')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3-methods.R
\name{original}
\alias{original}
\title{Get original data}
\usage{
original(x, ...)
}
\arguments{
\item{x}{Tibble.}

\item{...}{Additional arguments. None implemented.}
}
\value{
list
}
\description{
Get original data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/france.R
\name{france}
\alias{france}
\title{france}
\usage{
france(
  site,
  variable = "discharge",
  start_date = NULL,
  end_date = NULL,
  sites = FALSE,
  ...
)
}
\arguments{
\item{site}{French gauge number}

\item{variable}{Character. Either \code{stage} or \code{discharge}.}

\item{start_date}{Character. Optional start date with format
YYYY-MM-DD. Default is 1900-01-01.}

\item{end_date}{Character. End date with format YYYY-MM-DD.
Default is the current date.}

\item{sites}{Logical. If TRUE, returns a list of measurement
sites.}

\item{...}{Additional arguments. None implemented.}
}
\value{
data frame of discharge time-series
}
\description{
Retrieve French gauge data
}
\examples{
\dontrun{
df <- france('K027401001')
plot(df$Date, df$Q, type='l')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/germany.R
\name{germany}
\alias{germany}
\title{germany}
\usage{
germany(
  site,
  variable = "discharge",
  start_date = NULL,
  end_date = NULL,
  sites = FALSE,
  ...
)
}
\arguments{
\item{site}{German gauge id.}

\item{variable}{Character. Either \code{stage} or \code{discharge}.}

\item{start_date}{Character. Optional start date with format
YYYY-MM-DD. Default is 1900-01-01.}

\item{end_date}{Character. End date with format YYYY-MM-DD.
Default is the current date.}

\item{sites}{Logical. If TRUE, returns a list of measurement
sites.}

\item{...}{Additional arguments. None implemented.}
}
\value{
data frame of discharge time-series
}
\description{
Retrieve German gauge data.
}
\examples{
\dontrun{
# Note the German API currently only provides access to stage data from the previous 30 days.
site <- "593647aa-9fea-43ec-a7d6-6476a76ae868"
stage <- germany(site, variable = "stage")
plot(stage$Date, stage$H, type = "l")
}
}
